@@ -36,25 +36,25 @@
 // and then checked back in. You cannot make changes to these sections without
 // corresponding changes to the buildmeister script
 #ifndef NV_BUILD_BRANCH
-    #define NV_BUILD_BRANCH             r570_00
+    #define NV_BUILD_BRANCH             r573_48
 #endif
 #ifndef NV_PUBLIC_BRANCH
-    #define NV_PUBLIC_BRANCH             r570_00
+    #define NV_PUBLIC_BRANCH             r573_48
 #endif
 
 #if defined(NV_LINUX) || defined(NV_BSD) || defined(NV_SUNOS)
-#define NV_BUILD_BRANCH_VERSION         "rel/gpu_drv/r570/r570_00-407"
-#define NV_BUILD_CHANGELIST_NUM         (35817632)
+#define NV_BUILD_BRANCH_VERSION         "rel/gpu_drv/r570/r573_48-509"
+#define NV_BUILD_CHANGELIST_NUM         (36220323)
 #define NV_BUILD_TYPE                   "Official"
-#define NV_BUILD_NAME                   "rel/gpu_drv/r570/r570_00-407"
-#define NV_LAST_OFFICIAL_CHANGELIST_NUM (35817632)
+#define NV_BUILD_NAME                   "rel/gpu_drv/r570/r573_48-509"
+#define NV_LAST_OFFICIAL_CHANGELIST_NUM (36220323)
 
 #else     /* Windows builds */
-#define NV_BUILD_BRANCH_VERSION         "r570_00-404"
-#define NV_BUILD_CHANGELIST_NUM         (35811382)
-#define NV_BUILD_TYPE                   "Nightly"
-#define NV_BUILD_NAME                   "r570_00-250410"
-#define NV_LAST_OFFICIAL_CHANGELIST_NUM (35811869)
+#define NV_BUILD_BRANCH_VERSION         "r573_48-4"
+#define NV_BUILD_CHANGELIST_NUM         (36212872)
+#define NV_BUILD_TYPE                   "Official"
+#define NV_BUILD_NAME                   "573.55"
+#define NV_LAST_OFFICIAL_CHANGELIST_NUM (36212872)
 #define NV_BUILD_BRANCH_BASE_VERSION    R570
 #endif
 // End buildmeister python edited section
