@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2008-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2008-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -31,14 +31,16 @@
 
 // The "ONLY_IF => 'EXTDEV_GSYNC'," condition in sources.def is ignored!
 
-#include "nvRmReg.h"
+#include "nvrm_registry.h"
 #include "core/system.h"
 #include "core/locks.h"
 #include "gpu_mgr/gpu_mgr.h"
+#include "platform/sli/sli.h"
 #include "gpu/external_device/gsync.h"
 #include "os/os.h"
 #include "rmapi/control.h"
 #include "gpu/external_device/gsync_api.h"
+#include "gpu/external_device/dev_p2061.h"
 
 #include "gpu/external_device/dac_p2060.h"
 #include "gpu/external_device/dac_p2061.h"
@@ -153,7 +155,7 @@ gsyncmgrDestruct_IMPL(OBJGSYNCMGR *pGsyncMgr)
 // gsyncGetMasterableGpuByInstance
 // Return a gpu that can be timing master for the specified gsync instance.
 //
-POBJGPU
+OBJGPU *
 gsyncGetMasterableGpuByInstance(NvU32 gsyncInst)
 {
     OBJSYS *pSys = SYS_GET_INSTANCE();
@@ -241,7 +243,7 @@ gsyncP2060StartupProvider(OBJGSYNC *pGsync)
 
     // Constants to be returned in NV30F1_CTRL_GSYNC_GET_CAPS
     p2060->syncSkewResolutionInNs = NV_P2060_SYNC_SKEW_RESOLUTION;
-    p2060->syncSkewMax            = gsyncSupportsLargeSyncSkew_P2060(pExtDev) ? 
+    p2060->syncSkewMax            = gsyncSupportsLargeSyncSkew_P2060(pExtDev) ?
                                     NV_P2060_SYNC_SKEW_MAX_UNITS_FULL_SUPPORT :
                                     NV_P2060_SYNC_SKEW_MAX_UNITS_LIMITED_SUPPORT;
 
@@ -280,6 +282,12 @@ gsyncP2061StartupProvider(OBJGSYNC *pGsync)
         p2060->lastUserSkewSent            = NV_P2061_V204_SYNC_SKEW_INVALID;
     }
 
+    // Control5 sync mode settings are available for FW V3.00+
+    if (P2061_FW_REV(pGsync->pExtDev) >= 0x300)
+    {
+        pGsync->gsyncHal.gsyncSetRasterSyncDecodeMode  = gsyncSetRasterSyncDecodeMode_P2061_V300;
+    }
+
     return status;
 }
 
@@ -387,7 +395,19 @@ gsyncAttachGpu(PDACEXTERNALDEVICE pExtDev, OBJGPU *pGpu,
 
     pSys->setProperty(pSys, PDB_PROP_SYS_IS_GSYNC_ENABLED, NV_TRUE);
 
-    return gsyncStartupProvider(pGsync, externalDevice);
+    NV_ASSERT_OK_OR_RETURN(gsyncStartupProvider(pGsync, externalDevice));
+
+    if (pGsync->gpuCount == 1)
+    {
+        //
+        // Initialize the RasterSyncDecodeMode here.
+        // The timing source needs to agree with this, but all GPUs on the GSync
+        // should be the same, so we will use the type of the first GPU
+        //
+        NV_ASSERT_OK_OR_RETURN(pGsync->gsyncHal.gsyncSetRasterSyncDecodeMode(pGpu, pGpu, pGsync->pExtDev));
+    }
+
+    return NV_OK;
 }
 
 //
@@ -461,7 +481,7 @@ gsyncRemoveGpu(OBJGPU *pGpu)
 //
 // This routine returns a OBJGSYNC object associated with this GPU.
 //
-POBJGSYNC
+OBJGSYNC *
 gsyncmgrGetGsync(OBJGPU *pGpu)
 {
     OBJSYS *pSys = SYS_GET_INSTANCE();
@@ -1170,14 +1190,14 @@ gsyncIsAnyHeadFramelocked(OBJGSYNC *pGsync)
             {
                 // Check if assigned slaves displays are there.
                 if ((NV_OK == pGsync->gsyncHal.gsyncRefSlaves(pGpu,
-                     pGsync->pExtDev, refRead, &assigned, &refresh)) &&
+                     pGsync, refRead, &assigned, &refresh)) &&
                     (assigned != 0))
                 {
                     return NV_TRUE;
                 }
                 // Check if assigned master displays are there.
                 if ((NV_OK == pGsync->gsyncHal.gsyncRefMaster(pGpu,
-                     pGsync->pExtDev, refRead, &assigned, &refresh, NV_FALSE, NV_FALSE)) &&
+                     pGsync, refRead, &assigned, &refresh, NV_FALSE, NV_FALSE)) &&
                     (assigned != 0))
                 {
                     return NV_TRUE;
@@ -1333,7 +1353,7 @@ NV_STATUS gsyncGetStatusCaps(OBJGSYNC *pGsync,
 
     pGpu = gsyncGetMasterableGpu(pGsync);
 
-    status |= pGsync->gsyncHal.gsyncGetRevision(pGpu, pGsync->pExtDev, pParams);
+    status |= pGsync->gsyncHal.gsyncGetRevision(pGpu, pGsync, pParams);
 
     return status;
 }
@@ -1365,7 +1385,7 @@ gsyncSetLocalSync(OBJGSYNC *pGsync,
 {
     OBJGPU *pSourceGpu = NULL;
     OBJGPU *pTempGpu = NULL;
-    NvU8 i;
+    NvU32 i;
     NV_STATUS status = NV_OK;
 
     NV_ASSERT_OR_RETURN(pGsync && pGsync->pExtDev, NV_ERR_INVALID_DEVICE);
@@ -1456,12 +1476,12 @@ gsyncGetControlSync(OBJGSYNC *pGsync,
         // This is the case where we want to query what is current.
         if ( pParams->master )
         {
-            status |= pGsync->gsyncHal.gsyncRefMaster(pGpu, pGsync->pExtDev, refFetchGet,
+            status |= pGsync->gsyncHal.gsyncRefMaster(pGpu, pGsync, refFetchGet,
                 &pParams->displays, &pParams->refresh, NV_FALSE, NV_FALSE);
         }
         else
         {
-            status |= pGsync->gsyncHal.gsyncRefSlaves(pGpu, pGsync->pExtDev, refFetchGet,
+            status |= pGsync->gsyncHal.gsyncRefSlaves(pGpu, pGsync, refFetchGet,
                 &pParams->displays, &pParams->refresh);
         }
     }
@@ -1529,18 +1549,18 @@ gsyncSetControlSync(OBJGSYNC *pGsync,
         NvBool skipSwapBarrierWar = !!(pParams->configFlags &
             NV30F1_CTRL_GSYNC_GET_CONTROL_SYNC_CONFIG_FLAGS_KEEP_MASTER_SWAPBARRIER_DISABLED);
 
-        status |= pGsync->gsyncHal.gsyncRefMaster(pGpu, pGsync->pExtDev,
+        status |= pGsync->gsyncHal.gsyncRefMaster(pGpu, pGsync,
             refRead, &assigned, &refresh, NV_FALSE, NV_FALSE);
         pParams->displays |= assigned;
-        status |= pGsync->gsyncHal.gsyncRefMaster(pGpu, pGsync->pExtDev,
+        status |= pGsync->gsyncHal.gsyncRefMaster(pGpu, pGsync,
             refSetCommit, &pParams->displays, &pParams->refresh, NV_FALSE, skipSwapBarrierWar);
     }
     else
     {
-        status |= pGsync->gsyncHal.gsyncRefSlaves(pGpu, pGsync->pExtDev,
+        status |= pGsync->gsyncHal.gsyncRefSlaves(pGpu, pGsync,
             refRead, &assigned, &refresh);
         pParams->displays |= assigned;
-        status |= pGsync->gsyncHal.gsyncRefSlaves(pGpu, pGsync->pExtDev,
+        status |= pGsync->gsyncHal.gsyncRefSlaves(pGpu, pGsync,
             refSetCommit, &pParams->displays, &pParams->refresh);
     }
 
@@ -1583,18 +1603,18 @@ gsyncSetControlUnsync(OBJGSYNC *pGsync,
 
     if (pParams->master)
     {
-        status |= pGsync->gsyncHal.gsyncRefMaster(pGpu, pGsync->pExtDev,
+        status |= pGsync->gsyncHal.gsyncRefMaster(pGpu, pGsync,
             refRead, &assigned, &refresh, NV_FALSE, NV_FALSE);
         pParams->displays = assigned & ~pParams->displays;
-        status |= pGsync->gsyncHal.gsyncRefMaster(pGpu, pGsync->pExtDev,
+        status |= pGsync->gsyncHal.gsyncRefMaster(pGpu, pGsync,
             refSetCommit, &pParams->displays, &refresh, !!pParams->retainMaster, NV_FALSE);
     }
     else
     {
-        status |= pGsync->gsyncHal.gsyncRefSlaves(pGpu, pGsync->pExtDev,
+        status |= pGsync->gsyncHal.gsyncRefSlaves(pGpu, pGsync,
             refRead, &assigned, &refresh);
         pParams->displays = assigned & ~pParams->displays;
-        status |= pGsync->gsyncHal.gsyncRefSlaves(pGpu, pGsync->pExtDev,
+        status |= pGsync->gsyncHal.gsyncRefSlaves(pGpu, pGsync,
             refSetCommit, &pParams->displays, &refresh);
     }
 
@@ -2362,7 +2382,7 @@ static NV_STATUS
 gsyncNullRefMaster
 (
  OBJGPU       *pGpu,
- PDACEXTERNALDEVICE pExtDev,
+ OBJGSYNC *pGsync,
  REFTYPE rType,
  NvU32 *pDisplayMask,
  NvU32 *pRefresh,
@@ -2377,7 +2397,7 @@ static NV_STATUS
 gsyncNullRefSlaves
 (
  OBJGPU       *pGpu,
- PDACEXTERNALDEVICE pExtDev,
+ OBJGSYNC     *pGsync,
  REFTYPE rType,
  NvU32 *pDisplayMasks,
  NvU32 *pRefresh
@@ -2412,9 +2432,9 @@ gsyncNullSetWatchdog
 static NV_STATUS
 gsyncNullGetRevision
 (
- OBJGPU       *pGpu,
- PDACEXTERNALDEVICE pExtDev,
- GSYNCCAPSPARAMS *pParams
+    OBJGPU       *pGpu,
+    OBJGSYNC *pGsync,
+    GSYNCCAPSPARAMS *pParams
 )
 {
     return NV_ERR_GENERIC;
@@ -2486,6 +2506,35 @@ gsyncNullSetMulDiv
     return NV_ERR_NOT_SUPPORTED;
 }
 
+static NV_STATUS
+gsyncNullSetRasterSyncDecodeMode
+(
+    OBJGPU            *pGpu,
+    OBJGPU            *pServerGpu,
+    DACEXTERNALDEVICE *pExtDev
+)
+{
+    //
+    // Performs no action on the Gsync board except for P2061 FW 3.00+
+    // But, this detects when the raster sync mode is not supported by earlier FWs
+    //
+    NV2080_CTRL_INTERNAL_GSYNC_GET_RASTER_SYNC_DECODE_MODE_PARAMS
+            rasterSyncDecodeModeParams;
+    RM_API *pRmApi = GPU_GET_PHYSICAL_RMAPI(pServerGpu);
+
+    // Pre-3.00 FW can only use NV2080_CTRL_CMD_INTERNAL_GSYNC_GET_RASTER_SYNC_DECODE_MODE
+    NV_ASSERT_OK_OR_RETURN(pRmApi->Control(pRmApi, pServerGpu->hInternalClient,
+        pServerGpu->hInternalSubdevice, NV2080_CTRL_CMD_INTERNAL_GSYNC_GET_RASTER_SYNC_DECODE_MODE,
+        &rasterSyncDecodeModeParams, sizeof(rasterSyncDecodeModeParams)));
+
+    NV_CHECK_OR_RETURN(LEVEL_WARNING,
+        rasterSyncDecodeModeParams.rasterSyncDecodeMode ==
+        NV_P2061_CONTROL5_RASTER_SYNC_DECODE_MODE_VSYNC_SHORT_PULSE,
+        NV_ERR_INVALID_OPERATION);
+
+    return NV_OK;
+}
+
 static NV_STATUS
 gsyncSetupNullProvider(OBJGSYNCMGR *pGsyncMgr, NvU32 gsyncInst)
 {
@@ -2532,6 +2581,7 @@ gsyncSetupNullProvider(OBJGSYNCMGR *pGsyncMgr, NvU32 gsyncInst)
     pGsync->gsyncHal.gsyncSetHouseSyncMode     = gsyncNullSetHouseSyncMode;
     pGsync->gsyncHal.gsyncGetMulDiv            = gsyncNullGetMulDiv;
     pGsync->gsyncHal.gsyncSetMulDiv            = gsyncNullSetMulDiv;
+    pGsync->gsyncHal.gsyncSetRasterSyncDecodeMode = gsyncNullSetRasterSyncDecodeMode;
 
     return status;
 }
@@ -2665,24 +2715,24 @@ gsyncConvertNewEventToOldEventNum
 
 #ifdef DEBUG
 void
-gsyncDbgPrintGsyncEvents(NvU32 DebugLevel, NvU32 events, NvU32 iface)
+gsyncDbgPrintGsyncEvents(NvU32 events, NvU32 iface)
 {
     if (events & NVBIT(NV30F1_GSYNC_NOTIFIERS_SYNC_LOSS(iface)))
-        NV_PRINTF_EX(NV_PRINTF_MODULE, DebugLevel, "SYNC_LOSS ");
+        NV_PRINTF_EX(NV_PRINTF_MODULE, LEVEL_INFO, "SYNC_LOSS ");
     if (events & NVBIT(NV30F1_GSYNC_NOTIFIERS_SYNC_GAIN(iface)))
-        NV_PRINTF_EX(NV_PRINTF_MODULE, DebugLevel, "SYNC_GAIN ");
+        NV_PRINTF_EX(NV_PRINTF_MODULE, LEVEL_INFO, "SYNC_GAIN ");
     if (events & NVBIT(NV30F1_GSYNC_NOTIFIERS_STEREO_LOSS(iface)))
-        NV_PRINTF_EX(NV_PRINTF_MODULE, DebugLevel, "STEREO_LOSS ");
+        NV_PRINTF_EX(NV_PRINTF_MODULE, LEVEL_INFO, "STEREO_LOSS ");
     if (events & NVBIT(NV30F1_GSYNC_NOTIFIERS_STEREO_GAIN(iface)))
-        NV_PRINTF_EX(NV_PRINTF_MODULE, DebugLevel, "STEREO_GAIN ");
+        NV_PRINTF_EX(NV_PRINTF_MODULE, LEVEL_INFO, "STEREO_GAIN ");
     if (events & NVBIT(NV30F1_GSYNC_NOTIFIERS_HOUSE_GAIN))
-        NV_PRINTF_EX(NV_PRINTF_MODULE, DebugLevel, "HOUSE_GAIN ");
+        NV_PRINTF_EX(NV_PRINTF_MODULE, LEVEL_INFO, "HOUSE_GAIN ");
     if (events & NVBIT(NV30F1_GSYNC_NOTIFIERS_HOUSE_LOSS))
-        NV_PRINTF_EX(NV_PRINTF_MODULE, DebugLevel, "HOUSE LOSS ");
+        NV_PRINTF_EX(NV_PRINTF_MODULE, LEVEL_INFO, "HOUSE LOSS ");
     if (events & NVBIT(NV30F1_GSYNC_NOTIFIERS_RJ45_GAIN))
-        NV_PRINTF_EX(NV_PRINTF_MODULE, DebugLevel, "RJ45 GAIN ");
+        NV_PRINTF_EX(NV_PRINTF_MODULE, LEVEL_INFO, "RJ45 GAIN ");
     if (events & NVBIT(NV30F1_GSYNC_NOTIFIERS_RJ45_LOSS))
-        NV_PRINTF_EX(NV_PRINTF_MODULE, DebugLevel, "RJ45 LOSS ");
+        NV_PRINTF_EX(NV_PRINTF_MODULE, LEVEL_INFO, "RJ45 LOSS ");
 }
 #endif // DEBUG
 
