@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -23,7 +23,9 @@
 
 #include "kernel/gpu/fifo/kernel_channel.h"
 #include "kernel/mem_mgr/mem.h"
+#include "kernel/mem_mgr/os_desc_mem.h"
 #include "kernel/gpu/mmu/kern_gmmu.h"
+#include "platform/sli/sli.h"
 
 #include "published/volta/gv100/dev_pbdma.h"
 #include "rmapi/rs_utils.h"
@@ -170,7 +172,7 @@ kchannelCreateUserdMemDesc_GV100
     NvU32                   userdAddrLo;
     NvU32                   userdAddrHi;
     NvU32                   userdAlignment;
-    NvU32                   pageSize;
+    NvU64                   pageSize;
 
     NV_ASSERT_OR_RETURN(!gpumgrGetBcEnabledStatus(pGpu), NV_ERR_INVALID_STATE);
     pKernelChannel->pUserdSubDeviceMemDesc[gpumgrGetSubDeviceInstanceFromGpu(pGpu)] = NULL;
@@ -241,6 +243,16 @@ kchannelCreateUserdMemDesc_GV100
         return status;
     }
 
+    //
+    // For some memory types, just creating submem is not enough to ensure
+    // proper lifetimes. Make the channel dependant on this memory object,
+    // so that it gets released before memory itself.
+    //
+    if (dynamicCast(pUserdMemoryRef->pResource, OsDescMemory) != NULL)
+    {
+        refAddDependant(pUserdMemoryRef, RES_GET_REF(pKernelChannel));
+    }
+
     // check alignment
     if ((pUserdMemory->pMemDesc->Alignment < userdAlignment) &&
         (pUserdMemory->pMemDesc->Alignment  != 0))
