@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -30,7 +30,7 @@
 
 #include "core/core.h"
 #include "gpu/gpu.h"
-#include "gpu/mem_mgr/mem_mgr.h"
+#include "mem_mgr/mem.h"
 #include "gpu/mem_mgr/virt_mem_allocator_common.h"
 #include "gpu/mem_mgr/context_dma.h"
 #include "gpu/mem_mgr/mem_desc.h"
@@ -44,6 +44,7 @@
 #include "gpu/subdevice/subdevice.h"
 #include "rmapi/rs_utils.h"
 #include "rmapi/mapping_list.h"
+#include "platform/sli/sli.h"
 
 #include "gpu/bus/kern_bus.h"
 
@@ -83,7 +84,18 @@ _ctxdmaDestroyFBMappings
         //
         if (pCpuMapping)
         {
-            osUnmapPciMemoryKernelOld(pGpu, pContextDma->KernelVAddr[gpuSubDevInst]);
+            KernelBus *pKernelBus = GPU_GET_KERNEL_BUS(pGpu);
+
+            if(pGpu->getProperty(pGpu, PDB_PROP_GPU_COHERENT_CPU_MAPPING))
+            {
+                kbusUnmapCoherentCpuMapping_HAL(pGpu, pKernelBus, pContextDma->pMemDesc,
+                                                pContextDma->KernelVAddr[gpuSubDevInst],
+                                                pContextDma->KernelPriv);
+            }
+            else
+            {
+                osUnmapPciMemoryKernelOld(pGpu, pContextDma->KernelVAddr[gpuSubDevInst]);
+            }
             refRemoveMapping(pMemoryRef, pCpuMapping);
 
             ///
@@ -96,11 +108,10 @@ _ctxdmaDestroyFBMappings
             if (!IS_VIRTUAL(pGpu) && !IS_GSP_CLIENT(pGpu) &&
                 (pContextDma->FbApertureLen[gpuSubDevInst] != 0))
             {
-                KernelBus *pKernelBus = GPU_GET_KERNEL_BUS(pGpu);
-                kbusUnmapFbAperture_HAL(pGpu, pKernelBus, pContextDma->pMemDesc,
-                                        pContextDma->FbAperture[gpuSubDevInst],
-                                        pContextDma->FbApertureLen[gpuSubDevInst],
-                                        BUS_MAP_FB_FLAGS_MAP_UNICAST);
+                kbusUnmapFbApertureSingle(pGpu, pKernelBus, pContextDma->pMemDesc,
+                                          pContextDma->FbAperture[gpuSubDevInst],
+                                          pContextDma->FbApertureLen[gpuSubDevInst],
+                                          BUS_MAP_FB_FLAGS_MAP_UNICAST);
                 pContextDma->FbAperture[gpuSubDevInst]    = (NvU64)-1;
                 pContextDma->FbApertureLen[gpuSubDevInst] = 0;
             }
@@ -194,18 +205,18 @@ ctxdmaConstruct_IMPL
     pContextDma->Type                 = type;
     pContextDma->Limit                = limit;
 
-    for (i = 0; i < NV_ARRAY_ELEMENTS32(pContextDma->KernelVAddr); i++)
+    for (i = 0; i < NV_ARRAY_ELEMENTS(pContextDma->KernelVAddr); i++)
         pContextDma->KernelVAddr[i]   = NULL;
 
     pContextDma->KernelPriv           = NULL;
 
-    for (i = 0; i < NV_ARRAY_ELEMENTS32(pContextDma->FbAperture); i++)
+    for (i = 0; i < NV_ARRAY_ELEMENTS(pContextDma->FbAperture); i++)
     {
         pContextDma->FbAperture[i]    = (NvU64)-1;
         pContextDma->FbApertureLen[i] = 0;
     }
 
-    for (i = 0; i < NV_ARRAY_ELEMENTS32(pContextDma->Instance); i++)
+    for (i = 0; i < NV_ARRAY_ELEMENTS(pContextDma->Instance); i++)
     {
         pContextDma->Instance[i]      = 0;
         pContextDma->InstRefCount[i]  = 0;
@@ -441,7 +452,6 @@ _ctxdmaConstruct
     NV_STATUS           rmStatus        = NV_OK;
     Memory             *pMemory         = NULL;
     OBJGPU             *pGpu            = NULL;
-    MemoryManager      *pMemoryManager  = NULL;
     MEMORY_DESCRIPTOR  *pMemDesc        = NULL;
     NvHandle            hDevice         = 0;
     NvHandle            hClient         = pClient->hClient;
@@ -468,9 +478,20 @@ _ctxdmaConstruct
 
     gpuSetThreadBcState(pGpu, !pContextDma->bUnicast);
 
-    rmStatus = deviceGetByGpu(pClient, pGpu, NV_TRUE, &pDevice);
-    if (rmStatus != NV_OK)
-        return NV_ERR_INVALID_OBJECT_PARENT;
+    if (hSubDevice == 0)
+    {
+        //
+        // We verified that pMemory is parented by Device.
+        // pGpu == NULL & hSubdevice == 0 errors out above.
+        //
+        pDevice = pMemory->pDevice;
+    }
+    else
+    {
+        rmStatus = deviceGetByGpu(pClient, pGpu, NV_TRUE, &pDevice);
+        if (rmStatus != NV_OK)
+            return NV_ERR_INVALID_OBJECT_PARENT;
+    }
 
     pContextDma->pDevice = pDevice;
 
@@ -478,8 +499,6 @@ _ctxdmaConstruct
 
     API_GPU_FULL_POWER_SANITY_CHECK(pGpu, NV_TRUE, NV_FALSE);
 
-    pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
-
     pMemDesc = pMemory->pMemDesc;
 
     //
@@ -532,15 +551,6 @@ _ctxdmaConstruct
         }
     }
 
-    if (FLD_TEST_DRF(OS03, _FLAGS, _PTE_KIND, _BL, flags))
-    {
-        memdescSetPteKind(pContextDma->pMemDesc, memmgrGetPteKindBl_HAL(pGpu, pMemoryManager));
-    }
-    else if (FLD_TEST_DRF(OS03, _FLAGS, _PTE_KIND, _PITCH, flags))
-    {
-        memdescSetPteKind(pContextDma->pMemDesc, memmgrGetPteKindPitch_HAL(pGpu, pMemoryManager));
-    }
-
     //
     // If this ctxdma is a notifier AND it is in vidmem, create a kernel mapping to
     // it for use later in case a SW method needs to update it.
@@ -577,33 +587,54 @@ _ctxdmaConstruct
                         pContextDma->FbAperture[gpuSubDevInst] - pGpu->busInfo.gpuPhysFbAddr;
                 }
             }
+            else if (pGpu->getProperty(pGpu, PDB_PROP_GPU_COHERENT_CPU_MAPPING))
+            {
+                //
+                // Don't add the offset again, as the base address of pContextDma->pMemDesc
+                // already has the offset applied.
+                //
+                rmStatus = kbusMapCoherentCpuMapping_HAL(pGpu,
+                                                         pKernelBus,
+                                                         pContextDma->pMemDesc,
+                                                         0, // Offset already applied.
+                                                         pContextDma->Limit + 1,
+                                                         NV_PROTECT_READ_WRITE,
+                                                         &pContextDma->KernelVAddr[gpuSubDevInst],
+                                                         &pContextDma->KernelPriv);
+
+                if (rmStatus != NV_OK)
+                {
+                    SLI_LOOP_GOTO(done);
+                }
+            }
             else
             {
-                rmStatus = kbusMapFbAperture_HAL(pGpu, pKernelBus,
-                                                 pMemDesc, offset,
-                                                 &pContextDma->FbAperture[gpuSubDevInst],
-                                                 &pContextDma->FbApertureLen[gpuSubDevInst],
-                                                 BUS_MAP_FB_FLAGS_MAP_UNICAST, hClient);
-            }
-            if (rmStatus != NV_OK)
-            {
-                pContextDma->FbApertureLen[gpuSubDevInst] = 0;
-                SLI_LOOP_GOTO(done);
-            }
+                rmStatus = kbusMapFbApertureSingle(pGpu, pKernelBus,
+                                                   pMemDesc, offset,
+                                                   &pContextDma->FbAperture[gpuSubDevInst],
+                                                   &pContextDma->FbApertureLen[gpuSubDevInst],
+                                                   BUS_MAP_FB_FLAGS_MAP_UNICAST, pDevice);
 
-            memdescSetPageSize(pContextDma->pMemDesc, AT_GPU,
-                           memdescGetPageSize(pMemDesc, AT_GPU));
+                if (rmStatus != NV_OK)
+                {
+                    pContextDma->FbApertureLen[gpuSubDevInst] = 0;
+                    SLI_LOOP_GOTO(done);
+                }
 
-            rmStatus = osMapPciMemoryKernelOld(pGpu,
-                                              gpumgrGetGpuPhysFbAddr(pGpu) + pContextDma->FbAperture[gpuSubDevInst],
-                                              pContextDma->Limit+1,
-                                              NV_PROTECT_READ_WRITE,
-                                              &pContextDma->KernelVAddr[gpuSubDevInst],
-                                              NV_MEMORY_WRITECOMBINED);
-            if (rmStatus != NV_OK)
-            {
-                // Force out of the SLI loop
-                SLI_LOOP_BREAK;
+                memdescSetPageSize(pContextDma->pMemDesc, AT_GPU,
+                            memdescGetPageSize(pMemDesc, AT_GPU));
+
+                rmStatus = osMapPciMemoryKernelOld(pGpu,
+                                                gpumgrGetGpuPhysFbAddr(pGpu) + pContextDma->FbAperture[gpuSubDevInst],
+                                                pContextDma->Limit+1,
+                                                NV_PROTECT_READ_WRITE,
+                                                &pContextDma->KernelVAddr[gpuSubDevInst],
+                                                NV_MEMORY_WRITECOMBINED);
+                if (rmStatus != NV_OK)
+                {
+                    // Force out of the SLI loop
+                    SLI_LOOP_BREAK;
+                }
             }
 
             {
@@ -644,7 +675,7 @@ done:
             // host, if we are in guest OS (where IS_VIRTUAL(pGpu) is true),
             // do an RPC to the host to do the hardware update.
             //
-            NV_RM_RPC_ALLOC_CONTEXT_DMA(pGpu, hClient, hDevice, RES_GET_HANDLE(pContextDma), hClass, 
+            NV_RM_RPC_ALLOC_CONTEXT_DMA(pGpu, hClient, hDevice, RES_GET_HANDLE(pContextDma), hClass,
                                         flags, RES_GET_HANDLE(pMemory), offset, limit, rmStatus);
         }
     }
@@ -751,33 +782,6 @@ ctxdmaGetKernelVA_IMPL
     return NV_OK;
 }
 
-// ****************************************************************************
-//                            Deprecated Functions
-// ****************************************************************************
-
-/**
- * @warning This function is deprecated! Please use ctxdmaGetByHandle.
- */
-NV_STATUS
-CliGetContextDma
-(
-    NvHandle       hClient,
-    NvHandle       hContextDma,
-    ContextDma   **ppContextDma
-)
-{
-    RsClient   *pClient;
-    NV_STATUS   status;
-
-    *ppContextDma = NULL;
-
-    status = serverGetClientUnderLock(&g_resServ, hClient, &pClient);
-    if (status != NV_OK)
-        return NV_ERR_INVALID_CLIENT;
-
-    return ctxdmaGetByHandle(pClient, hContextDma, ppContextDma);
-}
-
 NV_STATUS
 ctxdmaMapTo_IMPL
 (
@@ -791,7 +795,7 @@ ctxdmaMapTo_IMPL
 
     //
     // For video memory, provide a way to look up the offset of an FB allocation within
-    // the given context target context dma. still useful for dFPGA.  
+    // the given context target context dma. still useful for dFPGA.
     // It is used by mods.
     //
     if ((memdescGetAddressSpace(memdescGetMemDescFromGpu(pSrcMemDesc, pGpu)) == ADDR_FBMEM) &&
