@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2015-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2015-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -40,7 +40,10 @@
 #if !defined(SRT_BUILD)
 // These files are not found on SRT builds
 #include "os/os.h"
+#define PMA_DEBUG 1
 #else
+// disable PMA debug prints on SRTs to not bloat the logs
+#define PMA_DEBUG 0
 NV_STATUS pmaNumaAllocate
 (
     PMA                    *pPma,
@@ -70,7 +73,7 @@ void pmaNumaSetReclaimSkipThreshold(PMA *pPma, NvU32 data)
 }
 #endif
 
-typedef NV_STATUS (*scanFunc)(void *, NvU64, NvU64, NvU64, NvU64, NvU64*, NvU64, NvU64, NvU64*, NvBool, NvBool);
+typedef NV_STATUS (*scanFunc)(void *, NvU64, NvU64, NvU64, NvU64, NvU64*, NvU64, NvU64, NvU64, NvU32, NvU64*, NvBool, NvBool);
 
 static void
 _pmaRollback
@@ -97,9 +100,10 @@ _pmaRollback
 
             for (j = 0; j < framesPerPage; j++)
             {
-                pPma->pMapInfo->pmaMapChangeState(pPma->pRegions[regId], (frameNum + j), oldState);
+                pPma->pMapInfo->pmaMapChangeStateAttrib(pPma->pRegions[regId], (frameNum + j), oldState, STATE_MASK);
             }
         }
+        pPma->pStatsUpdateCb(pPma->pStatsUpdateCtx, pPma->pmaStats.numFreeFrames);
     }
 
     if (failFrame != 0)
@@ -110,11 +114,22 @@ _pmaRollback
         frameNum = PMA_ADDR2FRAME(pPages[failCount], addrBase);
         for(i = 0; i < failFrame; i++)
         {
-            pPma->pMapInfo->pmaMapChangeState(pPma->pRegions[regId], (frameNum + i), oldState);
+            pPma->pMapInfo->pmaMapChangeStateAttrib(pPma->pRegions[regId], (frameNum + i), oldState, STATE_MASK);
         }
+        pPma->pStatsUpdateCb(pPma->pStatsUpdateCtx, pPma->pmaStats.numFreeFrames);
     }
 }
 
+static inline void
+_pmaDefaultStatsCallback
+(
+    void *pCtx,
+    NvU64 freeFrames
+)
+{
+    return;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 //
 // Public interfaces
@@ -208,10 +223,9 @@ pmaInitialize(PMA *pPma, NvU32 initFlags)
     //
     pMapInfo->pmaMapInit = pmaRegmapInit;
     pMapInfo->pmaMapDestroy = pmaRegmapDestroy;
-    pMapInfo->pmaMapChangeState = pmaRegmapChangeState;
     pMapInfo->pmaMapChangeStateAttrib = pmaRegmapChangeStateAttrib;
-    pMapInfo->pmaMapChangeStateAttribEx = pmaRegmapChangeStateAttribEx;
     pMapInfo->pmaMapChangePageStateAttrib = pmaRegmapChangePageStateAttrib;
+    pMapInfo->pmaMapChangeBlockStateAttrib = pmaRegmapChangeBlockStateAttrib;
     pMapInfo->pmaMapRead = pmaRegmapRead;
     pMapInfo->pmaMapScanContiguous = pmaRegmapScanContiguous;
     pMapInfo->pmaMapScanDiscontiguous = pmaRegmapScanDiscontiguous;
@@ -240,26 +254,6 @@ pmaInitialize(PMA *pPma, NvU32 initFlags)
         pPma->bNuma = !!(initFlags & PMA_INIT_NUMA);
 
         pPma->bNumaAutoOnline = !!(initFlags & PMA_INIT_NUMA_AUTO_ONLINE);
-
-        // If we want to run with address tree instead of regmap
-        if (initFlags & PMA_INIT_ADDRTREE)
-        {
-            pMapInfo->pmaMapInit = pmaAddrtreeInit;
-            pMapInfo->pmaMapDestroy = pmaAddrtreeDestroy;
-            pMapInfo->pmaMapChangeState = pmaAddrtreeChangeState;
-            pMapInfo->pmaMapChangeStateAttrib = pmaAddrtreeChangeStateAttrib;
-            pMapInfo->pmaMapChangeStateAttribEx = pmaAddrtreeChangeStateAttribEx;
-            pMapInfo->pmaMapChangePageStateAttrib = pmaAddrtreeChangePageStateAttrib;
-            pMapInfo->pmaMapRead = pmaAddrtreeRead;
-            pMapInfo->pmaMapScanContiguous = pmaAddrtreeScanContiguous;
-            pMapInfo->pmaMapScanDiscontiguous = pmaAddrtreeScanDiscontiguous;
-            pMapInfo->pmaMapGetSize = pmaAddrtreeGetSize;
-            pMapInfo->pmaMapGetLargestFree = pmaAddrtreeGetLargestFree;
-            pMapInfo->pmaMapScanContiguousNumaEviction = pmaAddrtreeScanContiguousNumaEviction;
-            pMapInfo->pmaMapGetEvictingFrames = pmaAddrtreeGetEvictingFrames;
-            pMapInfo->pmaMapSetEvictingFrames = pmaAddrtreeSetEvictingFrames;
-            NV_PRINTF(LEVEL_WARNING, "Going to use addrtree for PMA init!!\n");
-        }
     }
     pPma->pMapInfo = pMapInfo;
 
@@ -272,6 +266,9 @@ pmaInitialize(PMA *pPma, NvU32 initFlags)
     pPma->regSize = 0;
     portAtomicSetSize(&pPma->initScrubbing, PMA_SCRUB_INITIALIZE);
 
+    pPma->pStatsUpdateCtx = NULL;
+    pPma->pStatsUpdateCb = _pmaDefaultStatsCallback;
+
     // OK not to take lock since it's initialization
     NV_ASSERT(pmaStateCheck(pPma));
 
@@ -545,6 +542,8 @@ pmaRegisterRegion
         return status;
     }
 
+    pPma->pStatsUpdateCb(pPma->pStatsUpdateCtx, pPma->pmaStats.numFreeFrames);
+
     NV_PRINTF(LEVEL_INFO, "Registered region:\n");
     pmaRegionPrint(pPma, pPma->pRegDescriptors[id], pPma->pRegions[id]);
     NV_PRINTF(LEVEL_INFO, "%d region(s) now registered\n", pPma->regSize);
@@ -564,11 +563,15 @@ pmaAllocatePages
 {
     NvS32 regionList[PMA_REGION_SIZE];
     NV_STATUS status, prediction;
-    NvU32 flags, evictFlag, contigFlag, persistFlag, alignFlag, pinFlag, rangeFlag, blacklistOffFlag, partialFlag, skipScrubFlag, reverseFlag;
+    NvU32 flags;
+    NvBool evictFlag, contigFlag, persistFlag, alignFlag, pinFlag, rangeFlag, blacklistOffFlag, partialFlag, skipScrubFlag, reverseFlag;
+
     NvU32 regId, regionIdx;
     NvU64 numPagesAllocatedThisTime, numPagesLeftToAllocate, numPagesAllocatedSoFar;
     NvU64 addrBase, addrLimit;
     NvU64 rangeStart, rangeEnd;
+    NvU64 stride = 0;
+    NvU32 strideStart = 0;
     NvU64 *curPages;
     NvBool blacklistOffPerRegion[PMA_REGION_SIZE]={NV_FALSE};
     NvU64 blacklistOffAddrStart[PMA_REGION_SIZE] = {0}, blacklistOffRangeSize[PMA_REGION_SIZE] = {0};
@@ -578,7 +581,8 @@ pmaAllocatePages
     scanFunc useFunc;
     PMA_PAGESTATUS pinOption;
     NvU64 alignment = pageSize;
-    NvU32 framesPerPage  = (NvU32)(pageSize >> PMA_PAGE_SHIFT);
+    NvU32 framesPerPage;
+    NvU64 numFramesToAllocateTotal;
 
     //
     // A boolean indicating if we should try to evict. We at most try eviction once per call
@@ -587,8 +591,6 @@ pmaAllocatePages
     NvBool tryEvict = NV_TRUE;
     NvBool tryAlloc = NV_TRUE;
 
-    const NvU64 numFramesToAllocateTotal = framesPerPage * allocationCount;
-
     if (pPma == NULL || pPages == NULL || allocationCount == 0
         || (pageSize != _PMA_64KB && pageSize != _PMA_128KB && pageSize != _PMA_2MB && pageSize != _PMA_512MB)
         || allocationOptions == NULL)
@@ -626,6 +628,7 @@ pmaAllocatePages
             NV_PRINTF(LEVEL_ERROR, "Reverse allocation not supported on NUMA.\n");
             return NV_ERR_INVALID_ARGUMENT;
         }
+
         return pmaNumaAllocate(pPma, allocationCount, pageSize, allocationOptions, pPages);
     }
 
@@ -692,6 +695,9 @@ pmaAllocatePages
         }
     }
 
+    framesPerPage  = (NvU32)(pageSize >> PMA_PAGE_SHIFT);
+    numFramesToAllocateTotal = framesPerPage * allocationCount;
+
     pinOption = pinFlag ? STATE_PIN : STATE_UNPIN;
     pinOption |= persistFlag ? ATTRIB_PERSISTENT : 0;
 
@@ -719,7 +725,7 @@ pmaAllocatePages
         }
     }
 
-    tryEvict = (evictFlag == 1);
+    tryEvict = evictFlag;
 
 pmaAllocatePages_retry:
     //
@@ -727,10 +733,12 @@ pmaAllocatePages_retry:
     // after checking the scrubber so any pages allocated so far are not guaranteed
     // to be there any more. Restart from scratch.
     //
+#if PMA_DEBUG
     NV_PRINTF(LEVEL_INFO, "Attempt %s allocation of 0x%llx pages of size 0x%llx "
                           "(0x%x frames per page)\n",
                           contigFlag ? "contiguous" : "discontiguous",
                           (NvU64)allocationCount, pageSize, framesPerPage);
+#endif
 
     // Check if scrubbing is done before allocating each time before we retry
     if (bScrubOnFree)
@@ -816,7 +824,7 @@ pmaAllocatePages_retry:
 
         numPagesAllocatedThisTime = 0;
         status = (*useFunc)(pMap, addrBase, rangeStart, rangeEnd, numPagesLeftToAllocate,
-            curPages, pageSize, alignment, &numPagesAllocatedThisTime, !tryEvict, (NvBool)reverseFlag);
+            curPages, pageSize, alignment, stride, strideStart, &numPagesAllocatedThisTime, !tryEvict, reverseFlag);
 
         NV_ASSERT(numPagesAllocatedThisTime <= numPagesLeftToAllocate);
 
@@ -836,15 +844,6 @@ pmaAllocatePages_retry:
         curPages += numPagesAllocatedThisTime;
         numPagesLeftToAllocate -= numPagesAllocatedThisTime;
 
-        //
-        // PMA must currently catch addrtree shortcomings and fail the request
-        // Just follow the no memory path for now to properly release locks
-        //
-        if (status == NV_ERR_INVALID_ARGUMENT)
-        {
-            status = NV_ERR_NO_MEMORY;
-        }
-
         if (status == NV_ERR_IN_USE && !tryEvict)
         {
             //
@@ -1080,15 +1079,16 @@ pmaAllocatePages_retry:
             addrBase = pPma->pRegDescriptors[regId]->base;
             frameBase = PMA_ADDR2FRAME(pPages[0], addrBase);
 
+#if PMA_DEBUG
             NV_PRINTF(LEVEL_INFO, "Successfully allocated frames 0x%llx through 0x%llx\n",
                                   frameBase,
                                   frameBase + numFramesAllocated - 1);
+#endif
 
-            for (i = 0; i < numPagesAllocatedSoFar; i++)
-            {
-                pPma->pMapInfo->pmaMapChangePageStateAttrib(pMap, frameBase + (i * framesPerPage),
-                                                            pageSize, pinOption, NV_TRUE);
-            }
+            pPma->pMapInfo->pmaMapChangeBlockStateAttrib(pMap, frameBase, numPagesAllocatedSoFar * framesPerPage,
+                                                         pinOption, MAP_MASK);
+            
+            pPma->pStatsUpdateCb(pPma->pStatsUpdateCtx, pPma->pmaStats.numFreeFrames);
 
             if (blacklistOffFlag && blacklistOffPerRegion[regId])
             {
@@ -1102,45 +1102,61 @@ pmaAllocatePages_retry:
         }
         else
         {
-            NvU64 frameRangeStart = 0;
-            NvU64 lastFrameRangeEnd = 0;
             NvU64 frameBase = 0;
+#if PMA_DEBUG
+            NvU64 frameRangeStart   = 0;
+            NvU64 nextExpectedFrame = 0;
+            NvU32 frameRangeRegId   = 0;
 
             (void)frameRangeStart;   //Silence the compiler
-            (void)lastFrameRangeEnd;
+            (void)nextExpectedFrame;
+            (void)frameRangeRegId;
 
             NV_PRINTF(LEVEL_INFO, "Successfully allocated frames:\n");
+#endif
 
             for (i = 0; i < numPagesAllocatedSoFar; i++)
             {
                 regId = findRegionID(pPma, pPages[i]);
                 pMap  = pPma->pRegions[regId];
                 addrBase = pPma->pRegDescriptors[regId]->base;
-                frameBase = PMA_ADDR2FRAME(pPages[i], addrBase);
+                frameBase =  PMA_ADDR2FRAME(pPages[i], addrBase);
 
+#if PMA_DEBUG
                 // Print out contiguous frames in the same NV_PRINTF
                 if (i == 0)
                 {
                     frameRangeStart = frameBase;
+                    frameRangeRegId = regId;
                 }
-                else if ((lastFrameRangeEnd + 1) != frameBase)
+                else if ((frameRangeRegId != regId) || (nextExpectedFrame != frameBase))
                 {
                     // Break in frame range detected
-                    NV_PRINTF(LEVEL_INFO, "0x%llx through 0x%llx \n",
-                                          frameRangeStart,
-                                          lastFrameRangeEnd);
+                    NV_PRINTF(LEVEL_INFO, "0x%llx through 0x%llx region %d \n",
+                                          reverseFlag ? nextExpectedFrame + framesPerPage : frameRangeStart,
+                                          reverseFlag ? frameRangeStart + framesPerPage - 1 : nextExpectedFrame - 1,
+                                          frameRangeRegId);
 
                     frameRangeStart = frameBase;
+                    frameRangeRegId = regId;
                 }
-                lastFrameRangeEnd = frameBase + framesPerPage - 1;
+                nextExpectedFrame = reverseFlag ? frameBase - framesPerPage : frameBase + framesPerPage;
+#endif
 
-                pPma->pMapInfo->pmaMapChangePageStateAttrib(pMap, PMA_ADDR2FRAME(pPages[i], addrBase),
-                                                            pageSize, pinOption, NV_TRUE);
+                pPma->pMapInfo->pmaMapChangePageStateAttrib(pMap, frameBase,
+                                                            pageSize, pinOption, MAP_MASK);
 
             }
-            NV_PRINTF(LEVEL_INFO, "0x%llx through 0x%llx \n",
-                                  frameRangeStart,
-                                  frameBase + framesPerPage - 1);
+
+            pPma->pStatsUpdateCb(pPma->pStatsUpdateCtx, pPma->pmaStats.numFreeFrames);
+
+#if PMA_DEBUG
+                // Break in frame range detected
+                NV_PRINTF(LEVEL_INFO, "0x%llx through 0x%llx region %d \n",
+                                      reverseFlag ? nextExpectedFrame + framesPerPage : frameRangeStart,
+                                      reverseFlag ? frameRangeStart + framesPerPage - 1 : nextExpectedFrame - 1,
+                                      frameRangeRegId);
+#endif
         }
     }
 
@@ -1267,7 +1283,7 @@ pmaPinPages
             }
             else
             {
-                pPma->pMapInfo->pmaMapChangeState(pPma->pRegions[regId], (frameNum + j), STATE_PIN);
+                pPma->pMapInfo->pmaMapChangeStateAttrib(pPma->pRegions[regId], (frameNum + j), STATE_PIN, STATE_MASK);
             }
         }
     }
@@ -1278,59 +1294,6 @@ done:
     return status;
 }
 
-
-NV_STATUS
-pmaUnpinPages
-(
-    PMA      *pPma,
-    NvU64    *pPages,
-    NvLength  pageCount,
-    NvU64     pageSize
-)
-{
-    NvU32          framesPerPage, regId, i, j;
-    NvU64          frameNum, addrBase;
-    PMA_PAGESTATUS state;
-    framesPerPage  = (NvU32)(pageSize >> PMA_PAGE_SHIFT);
-
-    if (pPma == NULL || pageCount == 0 || pPages == NULL
-        || (pageSize != _PMA_64KB && pageSize != _PMA_128KB && pageSize != _PMA_2MB && pageSize != _PMA_512MB))
-    {
-        return NV_ERR_INVALID_ARGUMENT;
-    }
-
-    portSyncSpinlockAcquire(pPma->pPmaLock);
-
-    for(i = 0; i < pageCount; i++)
-    {
-        regId = findRegionID(pPma, pPages[i]);
-        addrBase = pPma->pRegDescriptors[regId]->base;
-        frameNum = PMA_ADDR2FRAME(pPages[i], addrBase);
-
-        for (j = 0; j < framesPerPage; j++)
-        {
-            state = pPma->pMapInfo->pmaMapRead(pPma->pRegions[regId], (frameNum + j), NV_FALSE);
-            if (state != STATE_PIN)
-            {
-                NV_PRINTF(LEVEL_ERROR, "Unpin failed at %dth page %dth frame\n",
-                                        i, j);
-                _pmaRollback(pPma, pPages, i, j, pageSize, STATE_PIN);
-                portSyncSpinlockRelease(pPma->pPmaLock);
-                return NV_ERR_INVALID_STATE;
-            }
-            else
-            {
-                pPma->pMapInfo->pmaMapChangeState(pPma->pRegions[regId], (frameNum + j), STATE_UNPIN);
-            }
-        }
-    }
-
-    portSyncSpinlockRelease(pPma->pPmaLock);
-
-    return NV_OK;
-}
-
-
 void
 pmaFreePages
 (
@@ -1369,8 +1332,6 @@ pmaFreePages
         return;
     }
 
-    framesPerPage = size >> PMA_PAGE_SHIFT;
-
     // Check if any scrubbing is done before we actually free
     if (bNeedScrub)
     {
@@ -1397,6 +1358,8 @@ pmaFreePages
 
     portSyncSpinlockAcquire(pPma->pPmaLock);
 
+    framesPerPage = size >> PMA_PAGE_SHIFT;
+
     for (i = 0; i < pageCount; i++)
     {
         regId    = findRegionID(pPma, pPages[i]);
@@ -1408,14 +1371,18 @@ pmaFreePages
         for (j = 0; j < framesPerPage; j++)
         {
             PMA_PAGESTATUS newStatus = (bScrubValid && bNeedScrub) ? ATTRIB_SCRUBBING : STATE_FREE;
+            PMA_PAGESTATUS exceptedMask = ATTRIB_EVICTING | ATTRIB_BLACKLIST;
+
             //
             // Reset everything except for the (ATTRIB_EVICTING and ATTRIB_BLACKLIST) state to support memory being freed
             // after being picked for eviction.
             //
-            pPma->pMapInfo->pmaMapChangeStateAttribEx(pPma->pRegions[regId], (frameNum + j), newStatus, ~(ATTRIB_EVICTING | ATTRIB_BLACKLIST));
+            pPma->pMapInfo->pmaMapChangeStateAttrib(pPma->pRegions[regId], (frameNum + j), newStatus, ~(exceptedMask));
         }
     }
 
+    pPma->pStatsUpdateCb(pPma->pStatsUpdateCtx, pPma->pmaStats.numFreeFrames);
+
     portSyncSpinlockRelease(pPma->pPmaLock);
 
     // Maybe we need to scrub the page on free
@@ -1424,17 +1391,18 @@ pmaFreePages
         PSCRUB_NODE pPmaScrubList = NULL;
         NvU64 count;
         if (scrubSubmitPages(pPma->pScrubObj, size, pPages, pageCount,
-                             &pPmaScrubList, &count) != NV_OK)
+                             &pPmaScrubList, &count) == NV_OK)
+        {
+            if (count > 0)
+            {
+                _pmaClearScrubBit(pPma, pPmaScrubList, count);
+            }
+        }
+        else
         {
             portAtomicSetSize(&pPma->scrubberValid, PMA_SCRUBBER_INVALID);
-            goto exit;
         }
 
-        if (count > 0)
-        {
-            _pmaClearScrubBit(pPma, pPmaScrubList, count);
-        }
-exit:
         // Free the actual list, although allocated by objscrub
         portMemFree(pPmaScrubList);
 
@@ -1464,6 +1432,8 @@ pmaClearScrubRange
             pmaSetBlockStateAttrib(pPma, physBase, physLimit - physBase + 1, 0, ATTRIB_SCRUBBING);
         }
     }
+
+    pPma->pStatsUpdateCb(pPma->pStatsUpdateCtx, pPma->pmaStats.numFreeFrames);
 }
 
 
@@ -1496,6 +1466,8 @@ pmaScrubComplete
         pmaSetBlockStateAttrib(pPma, physBase, physLimit - physBase + 1, 0, ATTRIB_SCRUBBING);
     }
 
+    pPma->pStatsUpdateCb(pPma->pStatsUpdateCtx, pPma->pmaStats.numFreeFrames);
+
     NV_PRINTF(LEVEL_INFO, "Inside\n");
     for (regionIdx = 0; regionIdx < pPma->regSize; regionIdx++)
     {
@@ -1508,6 +1480,19 @@ pmaScrubComplete
     return NV_OK;
 }
 
+void
+pmaRegisterUpdateStatsCb
+(
+    PMA *pPma,
+    pmaUpdateStatsCb_t pUpdateCb,
+    void *pCtxPtr
+)
+{
+    // Only supported right after init, so we don't bother taking locks.
+    pPma->pStatsUpdateCb = pUpdateCb;
+    pPma->pStatsUpdateCtx = pCtxPtr;
+    pUpdateCb(pCtxPtr, pPma->pmaStats.numFreeFrames);
+}
 
 NV_STATUS
 pmaRegisterEvictionCb
@@ -1618,6 +1603,24 @@ pmaGetFreeMemory
     NvU64           *pBytesFree
 )
 {
+#if !defined(SRT_BUILD)
+    NvU64 val;
+
+    portSyncSpinlockAcquire(pPma->pPmaLock);
+    NvBool nodeOnlined = pPma->nodeOnlined;
+    portSyncSpinlockRelease(pPma->pPmaLock);
+
+    if (nodeOnlined)
+    {
+        osGetNumaMemoryUsage(pPma->numaNodeId, pBytesFree, &val);
+        return;
+    }
+    //
+    // what to return when bNUMA == NV_TRUE and nodeOnlined==NV_FALSE?
+    // TODO : BUG 4199482.
+    //
+#endif
+
     portSyncSpinlockAcquire(pPma->pPmaLock);
 
     *pBytesFree = pPma->pmaStats.numFreeFrames << PMA_PAGE_SHIFT;
@@ -1638,6 +1641,24 @@ pmaGetTotalMemory
 
     *pBytesTotal = 0;
 
+#if !defined(SRT_BUILD)
+    NvU64 val;
+
+    portSyncSpinlockAcquire(pPma->pPmaLock);
+    NvBool nodeOnlined = pPma->nodeOnlined;
+    portSyncSpinlockRelease(pPma->pPmaLock);
+
+    if (nodeOnlined)
+    {
+        osGetNumaMemoryUsage(pPma->numaNodeId, &val, pBytesTotal);
+        return;
+    }
+    //
+    // what to return when bNUMA == NV_TRUE and nodeOnlined==NV_FALSE?
+    // TODO : BUG 4199482.
+    //
+#endif
+
     for (i = 0; i < pPma->regSize; i++)
     {
         pMap = pPma->pRegions[i];
@@ -1647,6 +1668,27 @@ pmaGetTotalMemory
     }
 }
 
+void
+pmaGetClientAddrSpaceSize
+(
+    PMA             *pPma,
+    NvU64           *pSize
+)
+{
+    NvU64 highestAddr = 0;
+
+    // Iterate the regions that were given to PMA
+    for (NvU32 i = 0; i < pPma->regSize; i++)
+    {
+        highestAddr = NV_MAX(highestAddr, pPma->pRegDescriptors[i]->limit);
+    }
+
+    // should end in 0xFFFF as a limit
+    NV_ASSERT((highestAddr & RM_PAGE_MASK) != 0);
+
+    *pSize = highestAddr + 1;
+}
+
 NV_STATUS
 pmaGetRegionInfo
 (
@@ -1818,6 +1860,8 @@ pmaGetClientBlacklistedPages
     NvBool bClientManagedBlacklist = NV_FALSE;
     PMA_BLACKLIST_CHUNK *pBlacklistChunks, *pBlacklistChunk;
 
+    NV_ASSERT_OR_RETURN_VOID((pPma != NULL) && (pChunks != NULL) && (pPageSize != NULL) && (pNumChunks != NULL));
+
     for (region = 0; region < pPma->regSize; region++)
     {
         pmaQueryBlacklistInfo(pPma, &blacklistCount,
