@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2021-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -44,7 +44,7 @@ _krcErrorWriteNotifierCpuMemHelper
 (
     OBJGPU    *pGpu,
     Memory    *pMemory,
-    NvHandle   hClient,
+    Device    *pDevice,
     NvU32      exceptType,
     RM_ENGINE_TYPE localRmEngineType,
     NV_STATUS  notifierStatus
@@ -56,7 +56,7 @@ _krcErrorWriteNotifierCpuMemHelper
     {
         case ADDR_VIRTUAL:
             notifyFillNotifierGPUVA(pGpu,
-                hClient,
+                pDevice,
                 RES_GET_HANDLE(pMemory),
                 memdescGetPhysAddr(pMemory->pMemDesc, AT_GPU_VA, 0),
                 exceptType,
@@ -103,7 +103,7 @@ krcErrorWriteNotifier_CPU
     NV_STATUS   status = NV_OK;
     ContextDma *pContextDma;
     Memory     *pMemory;
-    Device     *pDevice;
+    Device     *pDevice = GPU_RES_GET_DEVICE(pKernelChannel);
     //
     // Update the ECC error notifier for exceptTypes related to
     // CONTAINED/UNCONTAINED/DBE errors
@@ -113,14 +113,6 @@ krcErrorWriteNotifier_CPU
         exceptType == ROBUST_CHANNEL_UNCONTAINED_ERROR ||
         exceptType == ROBUST_CHANNEL_CONTAINED_ERROR);
 
-    status = deviceGetByInstance(RES_GET_CLIENT(pKernelChannel),
-                                 gpuGetDeviceInstance(pGpu),
-                                 &pDevice);
-    if (status != NV_OK)
-    {
-        return NV_ERR_INVALID_DEVICE;
-    }
-
     if (hypervisorIsVgxHyper() && bUpdateEccNotifier &&
         pKernelChannel->hEccErrorContext != NV01_NULL_OBJECT)
     {
@@ -147,7 +139,7 @@ krcErrorWriteNotifier_CPU
         {
             status = _krcErrorWriteNotifierCpuMemHelper(pGpu,
                 pMemory,
-                RES_GET_CLIENT_HANDLE(pKernelChannel),
+                pDevice,
                 exceptType,
                 localRmEngineType,
                 notifierStatus);
@@ -199,7 +191,7 @@ krcErrorWriteNotifier_CPU
     {
         status = _krcErrorWriteNotifierCpuMemHelper(pGpu,
             pMemory,
-            RES_GET_CLIENT_HANDLE(pKernelChannel),
+            pDevice,
             exceptType,
             localRmEngineType,
             notifierStatus);
@@ -249,6 +241,7 @@ NV_STATUS krcErrorSetNotifier_IMPL
 )
 {
     KernelFifo   *pKernelFifo     = GPU_GET_KERNEL_FIFO(pGpu);
+    OBJSYS       *pSys            = SYS_GET_INSTANCE();
     NvU32         status          = NV_OK;
     NvU32         flushFlags      = 0;
     NvBool        bNewListCreated = NV_FALSE;
@@ -258,6 +251,15 @@ NV_STATUS krcErrorSetNotifier_IMPL
     NV_ASSERT_OR_RETURN(!gpumgrGetBcEnabledStatus(pGpu), NV_ERR_INVALID_STATE);
     NV_ASSERT_OR_RETURN(pKernelChannel != NULL, NV_ERR_INVALID_CHANNEL);
 
+    //
+    // WAR bug 4503046: mark reboot required when any UVM channels receive an
+    // error.
+    //
+    if (pKernelChannel->bUvmOwned)
+    {
+        sysSetRecoveryRebootRequired(pSys, NV_TRUE);
+    }
+
     //
     // WAR bug 200326278, 200474671
     //
@@ -300,8 +302,8 @@ NV_STATUS krcErrorSetNotifier_IMPL
             KernelMIGManager *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
             MIG_INSTANCE_REF ref;
 
-            status = kmigmgrGetInstanceRefFromClient(pGpu, pKernelMIGManager,
-                                                     RES_GET_CLIENT_HANDLE(pKernelChannel),
+            status = kmigmgrGetInstanceRefFromDevice(pGpu, pKernelMIGManager,
+                                                     GPU_RES_GET_DEVICE(pKernelChannel),
                                                      &ref);
             if (status != NV_OK)
                 goto Error;
@@ -439,18 +441,22 @@ krcErrorSendEventNotifications_KERNEL
     KernelRc          *pKernelRc,
     KernelChannel     *pKernelChannel,
     RM_ENGINE_TYPE     rmEngineType,       // unused
+    NvU32              exceptLevel,        // unused
     NvU32              exceptType,
     RC_NOTIFIER_SCOPE  scope,
-    NvU16              partitionAttributionId
+    NvU16              partitionAttributionId,
+    NvBool             bOsRcCallbackNeeded // unused
 )
 {
     NV_ASSERT_OR_RETURN(!gpumgrGetBcEnabledStatus(pGpu), NV_ERR_INVALID_STATE);
-    NV_ASSERT_OR_RETURN(pKernelChannel != NULL, NV_ERR_INVALID_CHANNEL);
 
-    NV_ASSERT_OK_OR_RETURN(
-        krcErrorSendEventNotificationsCtxDma_HAL(pGpu, pKernelRc,
-                                                 pKernelChannel,
-                                                 scope));
+    if (pKernelChannel != NULL)
+    {
+        NV_ASSERT_OK_OR_RETURN(
+            krcErrorSendEventNotificationsCtxDma_HAL(pGpu, pKernelRc,
+                                                     pKernelChannel,
+                                                     scope));
+    }
 
     gpuNotifySubDeviceEvent(pGpu,
                             NV2080_NOTIFIERS_RC_ERROR,
